package eu.mhsl.craftattack.core.api.client;

import eu.mhsl.craftattack.core.Main;
import org.apache.http.client.utils.URIBuilder;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.function.Consumer;

@RepositoryLoader.IgnoreRepository
public abstract class HttpRepository extends Repository {
    private final Consumer<URIBuilder> baseUriBuilder;

    public HttpRepository(URI basePath) {
        this(basePath, null);
    }

    public HttpRepository(URI basePath, @Nullable Consumer<URIBuilder> baseUriBuilder) {
        super(basePath);

        this.baseUriBuilder = baseUriBuilder == null
            ? uriBuilder -> {
        }
            : baseUriBuilder;
    }

    protected <TInput, TOutput> ReqResp<TOutput> post(String command, TInput data, Class<TOutput> outputType) {
        return this.post(command, parameters -> {
        }, data, outputType);
    }

    protected <TInput, TOutput> ReqResp<TOutput> post(String command, Consumer<URIBuilder> parameters, TInput data, Class<TOutput> outputType) {
        HttpRequest request = this.getRequestBuilder(this.getUri(command, parameters))
            .POST(HttpRequest.BodyPublishers.ofString(this.gson.toJson(data)))
            .build();

        return this.execute(request, outputType);
    }

    protected <TOutput> ReqResp<TOutput> get(String command, Class<TOutput> outputType) {
        return this.get(command, parameters -> {
        }, outputType);
    }

    protected <TOutput> ReqResp<TOutput> get(String command, Consumer<URIBuilder> parameters, Class<TOutput> outputType) {
        HttpRequest request = this.getRequestBuilder(this.getUri(command, parameters))
            .GET()
            .build();

        return this.execute(request, outputType);
    }

    private URI getUri(String command, Consumer<URIBuilder> parameters) {
        try {
            URIBuilder builder = new URIBuilder(this.basePath + "/" + command);
            this.baseUriBuilder.accept(builder);
            parameters.accept(builder);
            return builder.build();
        } catch(URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    private HttpRequest.Builder getRequestBuilder(URI endpoint) {
        return HttpRequest.newBuilder()
            .uri(endpoint)
            .header("User-Agent", Main.instance().getServer().getBukkitVersion())
            .header("Content-Type", "application/json");
    }

    private <TResponse> ReqResp<TResponse> execute(HttpRequest request, Class<TResponse> clazz) {
        ReqResp<String> rawResponse = this.sendHttp(request);
        return new ReqResp<>(rawResponse.status(), this.gson.fromJson(rawResponse.data(), clazz));
    }

    private ReqResp<String> sendHttp(HttpRequest request) {
        try(HttpClient client = HttpClient.newHttpClient()) {
            this.validateThread(request.uri().getPath());
            HttpResponse<String> httpResponse = client.send(request, HttpResponse.BodyHandlers.ofString());
            return new ReqResp<>(httpResponse.statusCode(), httpResponse.body());
        } catch(IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

}
