package eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.displayName;

import eu.mhsl.craftattack.core.Main;
import eu.mhsl.craftattack.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.craftattack.appliances.gameplay.outlawed.Outlawed;
import eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.adminMarker.AdminMarker;
import eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.afkTag.AfkTag;
import eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.sleepTag.SleepTag;
import eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.yearRank.YearRank;
import eu.mhsl.craftattack.core.util.server.Floodgate;
import eu.mhsl.craftattack.core.util.text.ComponentUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.ComponentBuilder;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextColor;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.Objects;
import java.util.function.Supplier;
import java.util.logging.Level;

public class DisplayName extends Appliance {
    public interface Prefixed {
        @Nullable
        Component getNamePrefix(Player player);
    }

    public void update(Player player) {
        TextColor playerColor = this.queryAppliance(AdminMarker.class).getPlayerColor(player);
        List<Supplier<Prefixed>> prefixes = List.of(
            () -> this.queryAppliance(Outlawed.class),
            () -> this.queryAppliance(YearRank.class),
            () -> this.queryAppliance(AfkTag.class),
            () -> this.queryAppliance(SleepTag.class)
        );

        ComponentBuilder<TextComponent, TextComponent.Builder> playerName = Component.text();
        prefixes.stream()
            .map(prefixed -> prefixed.get().getNamePrefix(player))
            .filter(Objects::nonNull)
            .forEach(prefix -> playerName
                .append(prefix)
                .append(ComponentUtil.clearedSpace())
        );

        if(Floodgate.isBedrock(player)) {
            playerName
                .append(
                    Component.text("\uD83C\uDFAE", NamedTextColor.GRAY)
                        .hoverEvent(HoverEvent.showText(Component.text(
                            String.format("%s spielt die Minecraft: Bedrock Edition", player.getName())
                        )))
                )
                .append(ComponentUtil.clearedSpace());
        }

        playerName.append(Component.text(player.getName(), playerColor));

        this.setGlobal(player, playerName.build());
    }

    private void setGlobal(Player player, Component component) {
        try {
            player.customName(component);
            player.displayName(component);
            player.playerListName(component);
        } catch(Exception e) {
            Main.instance().getLogger().log(Level.SEVERE, e, e::getMessage);
        }
    }
}
